/*
 * See the files COPYING and README for copyright information and how to reach 
 * the author.
 *
 *  $Id: process-dvd.c,v 1.4 2004/12/05 18:50:34 lordjaxom Exp $
 */

#include "process-dvd.h"
#include "jobs.h"
#include "render.h"
#include "menuburn.h"
#include "config.h"
#include "setup.h"
#include "i18n.h"
#include <vdr/tools.h>
#include <vdr/interface.h>
#include <vdr/plugin.h>
#include <stdlib.h>

cBurnProcessDVD::cBurnProcessDVD(cBurnJob *Job):
		cBurnProcess(__DVD_COUNT__) {
	mJob = Job;
	mNumTracks = 0;
	mTrack = 0;
	mFactor = 0;
	mSizeMB = 0;
	mRender = new cBurnRender;

	mJob->SetTempDir(TempDir());
	SetLogfile("%s/dvd.log", TempDir());
	Start();
}

cBurnProcessDVD::~cBurnProcessDVD() {
	delete mRender;
}

void cBurnProcessDVD::Process(int Step, int SubStep, const char *Line) {
	int done, total, temp;
	float fdone;
	char *asp;
	
	switch ((eStepDVD)Step) {
	case sdVdrsync:
		if (sscanf(Line, "%d Mbytes of %d read", &done, &total) == 2) {
			if (total > 0 && done <= total)
				SetProgress(mJob->Count(), done * 100 / total);
		}
		break;

	case sdTcmplex:
		if (strn1cmp(Line, "*** Scanning video stream: 0%") == 0
				|| strn1cmp(Line, "*** Scanning audio stream: 0%") == 0
				|| strn1cmp(Line, "*** Multiplexing: 0%") == 0) {
			++mTrack;
			SetProgress(mJob->Count(), mTrack, mNumTracks + 1, 0);
		} else if (sscanf(Line, "*** Scanning video stream: %d%%", &done) == 1
				|| sscanf(Line, "*** Scanning audio stream: %d%%", &done) == 1
				|| sscanf(Line, "*** Multiplexing: %d%%", &done) == 1) {
			SetProgress(mJob->Count(), mTrack, mNumTracks + 1, done);
		}
		break;

	case sdDvdauthor:
		if (sscanf(Line, "STAT: fixing VOBU at %dMB (%d/%d, %d%%)", &temp, &temp,
				&temp, &done) == 4) {
			SetProgress(mJob->Count(), done);
		}
		break;

	case sdGrowisofs:
		if (sscanf(Line, "%f%% done, estimate finish %as", &fdone, &asp) == 2) {
			Dprintf("done: %f / %d \n", fdone, (int)fdone);
			SetBurning(true, (int)fdone);
			SetProgress(1, (int)fdone);
			free(asp);
		}
		break;

	default:
		break;
	}
}

bool cBurnProcessDVD::Continue(int Step, int SubStep) {
	cBurnRecording *rec;

	if (Step != sdGrowisofs && Status() == rsError)
		return false;

	switch ((eStepDVD)Step) {
	case sdVdrsync:
		if (SubStep < mJob->Count()) {
			rec = mJob->Get(SubStep);
			MakeDirectory("%s/VDRSYNC.%d", TempDir(), rec->Number());
			SetCmdline("vdrsync.pl -o %s/VDRSYNC.%d '%s'", TempDir(), rec->Number(),
					rec->FileName());
		} else
			SetNextStep();
		break;

	case sdRequant:
		if (SubStep == 0) {
			for (rec = mJob->First(); rec; rec = mJob->Next(rec))
				mSizeMB += rec->SizeMB();
			if (mSizeMB < DVDSIZE) {
				SetNextStep();
				break;
			} else
				mFactor = (mSizeMB * 1.1f) / DVDSIZE;
		}

		if (SubStep > 0) {
			const tTrackInfo *tracks;
			rec = mJob->Get(SubStep - 1);
			tracks = rec->GetTracks();
			Execute("rm -f %s", tracks[0].Filename);
			Execute("mv %s/requant.mpv %s", rec->TempDir(), tracks[0].Filename);
		}

		if (SubStep < mJob->Count()) {
			const tTrackInfo *tracks;
			double factor;
			double sizeMB;

			rec = mJob->Get(SubStep);
			tracks = rec->GetTracks();

			if (tracks[0].Type != ttVideo) {
				esyslog("BURN: No video tracks found");
				SetStatus(rsError);
				return false;
			}

			sizeMB = FileSize(tracks[0].Filename) / 1024.f / 1024.f;
			factor = sizeMB / (sizeMB - (rec->SizeMB() - rec->SizeMB() / mFactor));
			SetCmdline("requant %.2f < %s > %s/VDRSYNC.%d/requant.mpv", factor, 
					tracks[0].Filename, TempDir(), rec->Number());
		} else
			SetNextStep();
		break;

	case sdTcmplex:
		mTrack = -1;

		if (SubStep > 0) {
			rec = mJob->Get(SubStep - 1);
			Execute("rm -rf %s/*.m[p2]v", rec->TempDir());
			Execute("rm -rf %s/*.mp[a2]", rec->TempDir());
			Execute("rm -rf %s/*.ac3", rec->TempDir());
		}

		if (SubStep < mJob->Count()) {
			const tTrackInfo *tracks;

			rec = mJob->Get(SubStep);
			tracks = rec->GetTracks();

			if (tracks[0].Type != ttVideo) {
				esyslog("BURN: No video tracks found");
				SetStatus(rsError);
				return false;
			}
			if (tracks[1].Type == ttUnknown) {
				esyslog("BURN: No audio tracks found");
				SetStatus(rsError);
				return false;
			}

			for (mNumTracks = 0; tracks[mNumTracks].Type != ttUnknown; ++mNumTracks)
				;

			SetCmdline("tcmplex -i %s -p %s %s%s -m d -o %s/movie.mpg", 
					tracks[0].Filename, tracks[1].Filename,
					tracks[2].Type != ttUnknown ? "-s " : "",
					tracks[2].Type != ttUnknown ? tracks[2].Filename : "",
					rec->TempDir());
		} else
			SetNextStep();
		break;

	case sdDvdmenu:
		if (SubStep < mJob->Count()) {
			rec = mJob->Get(SubStep);
			mRender->SetTitleMenu(rec);
			SetProcedure(cBurnRender::RenderProc, mRender);
		} else if (SubStep == mJob->Count()) {
			mRender->SetMainMenu(mJob);
			SetProcedure(cBurnRender::RenderProc, mRender);
		} else
			SetNextStep();
		break;

	case sdDvdauthor:
		if (SubStep == 0) {
			cDvdauthorConfig conf(mJob, "%s/dvd.xml", TempDir());
			conf.SetTempDir(TempDir());
			conf.Write();
			MakeDirectory("%s/DVDAUTHOR", TempDir());
			SetCmdline("dvdauthor -x %s/dvd.xml", TempDir());
		} else {
			Execute("rm -rf %s/VDRSYNC.*", TempDir());
			SetNextStep();
		}
		break;
	
	case sdGrowisofs:
		if (SubStep == 0) {
			char *path;

			switch (mJob->StoreMode()) {
			case smCreateISO:
				SetCmdline("mkisofs -V VDR-BURN -dvd-video %s/DVDAUTHOR > '%s/%s.iso'", TempDir(), ISOdir, 
						     mJob->Title());
				break;
	
			case smBurnISO:
				SetCmdline("growisofs -use-the-force-luke=tty -dvd-compat -Z %s -V VDR-BURN -dvd-video %s/DVDAUTHOR",
				         DVDdev, TempDir());
				break;

			case smCreateBurn:
				asprintf(&path, "%s/fifo", TempDir());
				mkfifo(path, 0644);
				SetCmdline("growisofs -use-the-force-luke=tty -dvd-compat -Z %s=%s | mkisofs -V VDR-BURN -dvd-video "
				           "%s/DVDAUTHOR | tee '%s/%s.iso' > %s", DVDdev, path, TempDir(), ISOdir, mJob->Title(), path);
				free(path);
				break;
			}
			SetBurning(true);
		} else if (Status() == rsError) {
      switch (mJob->StoreMode()) {
			case smBurnISO:
			case smCreateBurn:
        Execute("eject %s", DVDdev);
        cBurnMenuBurn::Confirm(tr("Please insert an empty disk and press Ok"));
        SetNextStep(nsRetry);
        break;

			default:
				SetStatus(rsError);
				return false;
			}
		} else {
			SetBurning(false);
			SetNextStep();
		}
		break;

	default:
		break;
	}
	return true;
}
