/*
 * See the files COPYING and README for copyright information and how to reach 
 * the author.
 *
 *  $Id: process.c,v 1.3 2004/12/02 15:02:52 lordjaxom Exp $
 */

#include "process.h"
#include "pipe.h"
#include "menuburn.h"
#include "setup.h"
#include <vdr/tools.h>
#include <vdr/videodir.h>
#include <sys/types.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>

cBurnProcess::cBurnProcess(int NumSteps) {
	char *path;
	asprintf(&path, "%s/.vdr-burn.XXXXXX", Datadir);
	mStatus = rsNone;
	mCmdline = NULL;
	mProcedure = NULL;
	mProcData = NULL;
	mActive = false;
	mProgress = 0;
	mNumSteps = NumSteps;
	mCurStep = 0;
	mNextStep = nsContinue;
	mSubStep = -1;
	mBurning = false;
	mBurnProgress = 0;
	mTempDir = mkdtemp(path);
	mLogFile = NULL;
}

cBurnProcess::~cBurnProcess() {
	Stop();
	free(mCmdline);
	Execute("rm -rf %s", mTempDir);
	free(mTempDir);
}

void cBurnProcess::Stop(void) {
	if (mActive) {
		mActive = false;
#if VDRVERSNUM < 10314
		WakeUp();
#endif
		Cancel(3);
	}
}

void cBurnProcess::SetProgress(int SubSteps, int Percent) {
	mProgress = mCurStep * 100 / mNumSteps 
			+ mSubStep * 100 / mNumSteps / SubSteps 
			+ Percent / mNumSteps / SubSteps;
}

void cBurnProcess::SetProgress(int SubSteps, int Subsub, int Subsubs, 
		int Percent) {
	mProgress = mCurStep * 100 / mNumSteps 
			+ mSubStep * 100 / mNumSteps / SubSteps 
			+ Subsub * 100 / mNumSteps / SubSteps / Subsubs 
			+ Percent / mNumSteps / SubSteps / Subsubs;
}

void cBurnProcess::SetCmdline(const char *Format, ...) {
	va_list ap;
	va_start(ap, Format);
	vasprintf(&mCmdline, Format, ap);
	va_end(ap);
}

void cBurnProcess::SetLogfile(const char *Format, ...) {
	char *path;
	va_list ap;
	if (mLogFile)
		fclose(mLogFile);
	va_start(ap, Format);
	vasprintf(&path, Format, ap);
	va_end(ap);
	mLogFile = fopen(path, "w");
	free(path);
}

void cBurnProcess::Execute(const char *Format, ...) {
	char *cmd;
	va_list ap;
	va_start(ap, Format);
	vasprintf(&cmd, Format, ap);
	va_end(ap);
	if (mLogFile) {
		fprintf(mLogFile, "++ executing: sh -c '%s'\n", cmd);
		fflush(mLogFile);
	}
	Dprintf("executing: sh -c '%s'\n", cmd);
	SystemExec(cmd);
	free(cmd);
}

void cBurnProcess::MakeDirectory(const char *Format, ...) {
	char *path;
	va_list ap;
	va_start(ap, Format);
	vasprintf(&path, Format, ap);
	va_end(ap);
	mkdir(path, 0700);
	free(path);
}

bool cBurnProcess::HasNext(void) {
	mNextStep = nsContinue;
	++mSubStep;
	if (Continue(mCurStep, mSubStep)) {
		switch (mNextStep) {
		case nsNext:
			SetProgress(mSubStep + 1, 100);
			++mCurStep;
		case nsRetry:
			mSubStep = -1;
			break;

		case nsContinue:
			break;
		}
		return mCurStep < mNumSteps;
	}
	return false;
}

void cBurnProcess::PutLine(const char *Line) {
	if (mLogFile) {
		fprintf(mLogFile, "%s\n", Line);
		fflush(mLogFile);
	}
	Process(mCurStep, mSubStep, Line);
}

void cBurnProcess::Action(void) {
	mActive = true;
	isyslog("BURN: Subprocess watcher started (pid=%d)", getpid());

	while (mActive && HasNext()) {
		cBurnPipe pipe;

		mStatus = rsNone;

		if (mCmdline == NULL && mProcedure == NULL)
			continue;

		pipe.SetWorkdir(mTempDir);
		if (mCmdline != NULL) {
			if (!pipe.Launch(mCmdline)) {
				mActive = false;
				break;
			}
			if (mLogFile) {
				fprintf(mLogFile, " ++ started: sh -c '%s'\n", mCmdline);
				fflush(mLogFile);
			}
		} else if (mProcedure != NULL) {
			if (!pipe.Launch(mProcedure, mProcData)) {
				mActive = false;
				break;
			}
			if (mLogFile) {
				fprintf(mLogFile, " ++ started an internal procedure\n");
				fflush(mLogFile);
			}
		}
		free(mCmdline); 
		mCmdline = NULL;
		mProcedure = NULL;
	
		while (mActive) {
			char *line;
			if (!pipe.GetLine(line)) {
				mActive = false;
				break;
			}
			if (line) {
				PutLine(line);
				free(line);
			} else
				break;
		}

		if (!pipe.Exit(mActive)) {
			mStatus = rsError;
			//mActive = false;
		}
	}

	if (mActive) {
		mActive = false;
		mProgress = 100;
	}

	if (mLogFile) {
		fclose(mLogFile);
		mLogFile = NULL;
	}

	isyslog("BURN: Subprocess watcher stopped");
			
	cBurnMenuBurn::NextJob();
}

